clc
clear all
close all

%%%% Computing the ACT for an AR1
%%%%

% parameters
k0     = 1;
sig_sq = 1;
rho    = 1;

VN   = [500 1000:1000:5000]; %range of values for N
Iter = 1000               ;

ACT = [];
SD  = [];

for N = VN

N

ACT1 = [];
ACT2 = [];
ACT3 = [];

for i = 1:Iter

% random white noise
sig = sig_sq^0.5;
eps = sig*randn(N,1);

% lambda series
lambda = [];
temp   = 0;
for i = 1:N
    temp1  = rho*temp+eps(i);
    lambda = [lambda temp1];
    temp   = temp1;
end

x      = lambda;
Av_x   = sum(x)/length(x);
Sig2_x = sum((x-Av_x).^2)/length(x);


% calculate ACT
temp = 0;

if x(1) > Av_x
    temp = 1;
end

TvecA = [];
TvecB = [];
T1    = 1 ;

for j = 2:N
    if temp == 0
        if x(j) > Av_x
            temp = 1;
            TvecB = [TvecB j-T1];
            T1 = j;
        end
    end
    
    if temp == 1
        if x(j) < Av_x
            temp = 0;
            TvecA = [TvecA j-T1];
            T1 = j;
        end
    end
end 
    
ACT1 = [ACT1 (sum(TvecA)+sum(TvecB))/(length(TvecA)+length(TvecB))];
ACT2 = [ACT2 N/(length(TvecA)+length(TvecB))];
ACT3 = [ACT3 N/(1+length(TvecA)+length(TvecB))]  ;

end

ACT = [ACT [mean(ACT1), mean(ACT2), mean(ACT3)]'];
SD  = [SD std((ACT1+ACT2)/2)/Iter^0.5];


end

% Analytical ACT
AACT = (pi./(pi/2-atan(rho./((1-rho.^2).^0.5))))*ones(1,length(VN));

figure
hold on

plot(VN, ACT(1,:), 'k--', 'LineWidth',2)
plot(VN, ACT(2,:), 'r--', 'LineWidth',2)
% plot(VN, ACT(3,:), 'b--', 'LineWidth',2)

plot(VN, (ACT(1,:)+ACT(2,:))./2, 'y--', 'LineWidth',2)

plot(VN, AACT, 'g--', 'LineWidth',2)

title('ACT for AR1','FontSize',16,'Interpreter','Latex')
xlabel('$N$','FontSize',16,'Interpreter','Latex')
ylabel('ACT','FontSize',16,'Interpreter','Latex')
legend('Method1','Method2', 'Method 1+2', 'Analytical', 'Location', 'SE','Interpreter','Latex')

grid on

hold off

figure
hold on

plot(VN, SD, 'k', 'LineWidth',1.5)

%title('SD of ACT AR1','FontSize',16,'Interpreter','Latex')
xlabel('$N$','FontSize',16,'Interpreter','Latex')
ylabel('SE','FontSize',16,'Interpreter','Latex')

grid on

hold off




