clc
clear all
close all

%%%% Computing the ACT for an AR1 both numerically and theoretically
%%%%

% parameters
k0     = 1;
sig_sq = 1;

N      = 10^4; %length of time series
Iter = 100;

Vrho = 0:0.05:0.98; %range of values for rho
ACT  = [];

for rho = [0:0.05:0.98]

rho

ACT1 = [];
ACT2 = [];

for ii = 1:Iter

% random white noise
sig = sig_sq^0.5;
eps = sig*randn(N,1);

% lambda series
lambda = [];
temp   = 0;
for i = 1:N
    temp1  = rho*temp+eps(i);
    lambda = [lambda temp1];
    temp   = temp1;
end

x = lambda;
Av_x = sum(x)/length(x);
Sig2_x = sum((x-Av_x).^2)/length(x);

temp = 0;

if x(1) > Av_x
    temp = 1;
end

TvecA = [];
TvecB = [];
T1    = 1 ;

for j = 2:N
    if temp == 0
        if x(j) > Av_x
            temp = 1;
            TvecB = [TvecB j-T1];
            T1 = j;
        end
    end
    
    if temp == 1
        if x(j) < Av_x
            temp = 0;
            TvecA = [TvecA j-T1];
            T1 = j;
        end
    end
end 
    
Temp1  = (sum(TvecA)+sum(TvecB))/(length(TvecA)+length(TvecB));

nn     = length(x);

if length(TvecA) == 0

TvecA = [nn-1-TvecB(1)];

end

if length(TvecB) == 0

TvecB = [nn-1-TvecA(1)];

end

ACT1 = [ACT1 (sum(TvecA)+sum(TvecB))/(length(TvecA)+length(TvecB))];
ACT2 = [ACT2 N/(length(TvecA)+length(TvecB))];

end

ACT  = [ACT (mean(ACT1)+mean(ACT2))/2];

end

% Analytical ACT
AACT  = pi./(pi/2-atan(Vrho./((1-Vrho.^2).^0.5)));
AACT2 = 2./(1-Vrho);

figure
hold on

plot(Vrho, ACT, 'k', 'LineWidth',1.5)
plot(Vrho, AACT, 'r--', 'LineWidth',1.5)
plot(Vrho, AACT2, 'g--', 'LineWidth',1.5)


%title('ACT for AR1','FontSize',16,'Interpreter','Latex')
xlabel('$\rho$','FontSize',16,'Interpreter','Latex')
ylabel('ACT, $\widehat{ACT}$','FontSize',16,'Interpreter','Latex')
legend('Simulation','Analytical','Location','SE','Interpreter','Latex')
grid on

hold off






